package sitebuilder

import (
	"context"
	"lmika.dev/lmika/hugo-cms/models"
)

func (s *Service) Publish(site models.Site) models.Job {
	return models.Job{
		Do: func(ctx context.Context) error {
			s.signalSiteBuildingStarted(ctx, site)
			defer s.signalSiteBuildingFinished(ctx, site)

			return s.publish(ctx, site)
		},
	}
}

func (s *Service) publish(ctx context.Context, site models.Site) error {
	if _, err := s.hugo.PreviewSite(ctx, site); err != nil {
		return err
	}

	targets, err := s.db.GetPublishTargets(ctx, site.ID)
	if err != nil {
		return err
	}

	for _, target := range targets {
		if err := s.publishTarget(ctx, site, target); err != nil {
			return err
		}
	}
	return nil
}

func (s *Service) publishTarget(ctx context.Context, site models.Site, target models.PublishTarget) error {
	outDir, err := s.hugo.PublishSite(ctx, site, target)
	if err != nil {
		return err
	}

	return s.netlify.Publish(ctx, target, outDir)
}
